from functools import wraps

from django.contrib.auth import authenticate, get_user_model, login
from django.utils.translation import ugettext_lazy as _

from promise import Promise, is_thenable

from . import exceptions
from .shortcuts import get_token


def token_auth(f):
    @wraps(f)
    def wrapper(root, info, password, **kwargs):
        def on_resolve(value):
            user, payload = value
            payload['token'] = get_token(user)
            return payload

        username = kwargs.get(get_user_model().USERNAME_FIELD)
        user = authenticate(username=username, password=password)

        if user is None:
            raise exceptions.GraphQLJWTError(
                _('Please, enter valid credentials'))

        login(info.context.get('request'), user)
        result = f(root, info, **kwargs)
        value = (user, result)
        # Improved mutation with thenable check
        if is_thenable(result):
            return Promise.resolve(value).then(on_resolve)
        return on_resolve(value)

    return wrapper



from .mutations import resolve_verify, resolve_refresh, resolve_token_auth, jwt_schema
from .scalar import GenericScalar

# from .mutations import
__all__ = ['resolve_verify', 'resolve_refresh', 'resolve_token_auth', 'jwt_schema', 'GenericScalar', ]



from calendar import timegm
from datetime import datetime

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from promise import Promise

from . import settings
from .decorators import token_auth
from .shortcuts import get_token
from .utils import get_payload, get_user_by_payload

__all__ = ['resolve_verify', 'resolve_refresh', 'resolve_token_auth', 'jwt_schema']

jwt_schema = '''
    scalar GenericScalar
    
    type VerifyToken {
        payload: GenericScalar
    }
    type RefreshToken {
        token: String
        payload: GenericScalar
    }
    type TokenAuth {
        token: String
    }
'''


def resolve_verify(obj, info, token, **kwargs):
    return {'payload': get_payload(token)}


def resolve_refresh(obj, info, token, **kwargs):
    payload = get_payload(token)
    user = get_user_by_payload(payload)
    orig_iat = payload.get('orig_iat')

    if orig_iat:
        utcnow = timegm(datetime.utcnow().utctimetuple())
        expiration = orig_iat + settings.JWT_REFRESH_EXPIRATION_DELTA.total_seconds()

        if utcnow > expiration:
            raise ValidationError(_('RefreshToken has expired'))
    else:
        raise ValidationError(_('orig_iat field is required'))

    token = get_token(user, orig_iat=orig_iat)

    return {'token': token, 'payload': payload}


@token_auth
def resolve_token_auth(obj, info, **kwargs):
    return {}



from ariadne.scalars import ScalarType
from graphql.type.scalars import MAX_INT, MIN_INT
from graphql import StringValueNode, BooleanValueNode, IntValueNode, FloatValueNode, ListValueNode, ObjectValueNode

GenericScalar = ScalarType('GenericScalar')


@GenericScalar.serializer
@GenericScalar.value_parser
def identity(value):
    return value


@GenericScalar.literal_parser
def parse_literal(ast):
    if isinstance(ast, (StringValueNode, BooleanValueNode)):
        return ast.value
    elif isinstance(ast, IntValueNode):
        num = int(ast.value)
        if MIN_INT <= num <= MAX_INT:
            return num
    elif isinstance(ast, FloatValueNode):
        return float(ast.value)
    elif isinstance(ast, ListValueNode):
        return [parse_literal(value) for value in ast.values]
    elif isinstance(ast, ObjectValueNode):
        return {
            field.name.value: parse_literal(field.value)
            for field in ast.fields
        }
    else:
        return None
